import React, { Component } from 'react';
import { AuctionList } from './AuctionList';
import { AuctionDetails } from './AuctionDetails';
import { NewAuction, NewAuctionInfo } from './NewAuction';
import { Auction } from './auction';


/**
 * Indicates which page to show. If it is the details page, the argument
 * includes the specific auction to show the details of.
 */
type Page = {kind: "list"} | {kind: "new"} | {kind: "details", index: number};

type AppState = {
  /** Current page being displayed. */
  page: Page;

  /** Current active auctions. */
  auctions: ReadonlyArray<Auction>;
};

// Enable debug logging of events in this component.
const DEBUG: boolean = false;

/** Top-level component that displays the appropriate page of the Auction App. */
export class App extends Component<{}, AppState> {

  constructor(props: {}) {
    super(props);

    this.state = {page: {kind: "list"}, auctions: []};
  }

  render = (): JSX.Element => {
    if (this.state.page.kind === "list") {
      if (DEBUG) console.debug("rendering list page");
      return <AuctionList auctions={this.state.auctions}
                          onNewClick={this.doNewClick}
                          onAuctionClick={this.doAuctionClick}/>;

    } else if (this.state.page.kind === "new") {
      if (DEBUG) console.debug("rendering add page");
      return <NewAuction onStartClick={this.doStartClick}
                         onBackClick={this.doBackClick}/>;

    } else {  // details
      const index = this.state.page.index;
      const auction = this.state.auctions[index];
      if (DEBUG) console.debug(`rendering details page for "${auction.name}"`);
      return <AuctionDetails auction={auction}
                  onBidClick={(bidder, amt) => this.doBidClick(index, bidder, amt)}
                  onBackClick={this.doBackClick}/>;
    }
  };

  // Handles the onNewClick callback. Navigates to the new auction page.
  doNewClick = (): void => {
    if (DEBUG) console.debug("set state to new");
    this.setState({page: {kind: "new"}});
  };

  // Handles the onAuctionClick callback. Navigates to the details page for
  // the auction that corresponds to the given index.
  doAuctionClick = (index: number): void => {
    const auction = this.state.auctions[index];
    if (DEBUG) console.debug(`set state to details for "${auction.name}"`);
    this.setState({page: {kind: "details", index}});
  };

  // Handles the onStartClick callback. Navigates to the main/list page and
  // starts/adds a new auction with the given info to the list of auctions.
  doStartClick = (info: NewAuctionInfo): void => {
    if (DEBUG) console.debug("add an auction and then show the full list");

    // Act as if the seller bid just below the min bid, this requires
    // someone else to bid at least the min bid to win the auction.
    const maxBid = info.minBid - 1;
    const maxBidder = info.seller;

    // Record the time since epoch when this ends.
    const endTime = Date.now() + info.minutes * 60 * 1000;

    const auction = {
        name: info.name, description: info.description, seller: info.seller,
        maxBid, maxBidder, endTime
      };
    const auctions = this.state.auctions.concat([auction]);
    this.setState({page: {kind: "list"}, auctions});
  };

  // Handles the onBackClick callback. Navigates to the main/list page.
  doBackClick = (): void => {
    if (DEBUG) console.debug("set state to list");
    this.setState({page: {kind: "list"}});
  };

  // Handles the onBidClick callback. Re-navigates to the details page for the
  // auction with the given index and updated bidder and amount.
  doBidClick = (index: number, bidder: string, amount: number): void => {
    const oldAuctionVal = this.state.auctions[index];
    const newAuctionVal = {
        name: oldAuctionVal.name, seller: oldAuctionVal.seller,
        description: oldAuctionVal.description, endTime: oldAuctionVal.endTime,
        maxBidder: bidder, maxBid: amount
      };

    // Re-display the details page with the new auction.
    const auctions = this.state.auctions.slice(0, index)
        .concat([newAuctionVal])
        .concat(this.state.auctions.slice(index+1));
    this.setState({auctions, page: {kind: "details", index}});
  };
}
