import React, { Component, MouseEvent } from 'react';
import { Auction } from './auction';


type ListProps = {
  /** Current active auctions. */
  auctions: ReadonlyArray<Auction>;

  /** Tell the parent that new auction was selected (to be displayed). */
  onNewClick: () => void;

  /**
   * Tell the parent that the auction belonging to the given index was selected
   * (to be displayed).
   */
  onAuctionClick: (index: number) => void;
};

type ListState = {
  /** Current time when rendering. */
  now: number
};


/** Displays the UI for the list of auctions. */
export class AuctionList extends Component<ListProps, ListState> {
  constructor(props: ListProps) {
    super(props);

    this.state = {now: Date.now()};
  }

  render = (): JSX.Element => {
    return (
      <div>
        <h2>Current Auctions</h2>
        <ul>{this.renderAuctions()}</ul>
        <button type="button" onClick={this.doRefreshClick}>Refresh</button>
        <button type="button" onClick={this.doNewClick}>New Auction</button>
      </div>);
  };

  // Returns a li for each auction in the auctions list, with each li containing
  // a link to the auction item's details and the amount of time remaining before
  // the auction ends.
  renderAuctions = (): JSX.Element[] => {
    const auctions: JSX.Element[] = [];
    // Inv: auctions = auctions contains a li for each auction of auctions[0 .. i-1]
    for (const auction of this.props.auctions) {
      const i = this.props.auctions.findIndex((i) => i === auction);

      const min = (auction.endTime - this.state.now) / 60 / 1000;
      const desc = (min < 0) ? "" :
          <span>&nbsp;&ndash; {Math.round(min)} minutes remaining</span>;
      auctions.push(
        <li key={auction.name}>
          <a href="#" onClick={(evt) => this.doAuctionClick(evt, i)}>{auction.name}</a>
          {desc}
        </li>);
    }
    return auctions;
  };

  // Called when the user clicks the refresh button. Updates the time of the
  // auction items.
  doRefreshClick = (_: MouseEvent<HTMLButtonElement>): void => {
    this.setState({now: Date.now()});
  };

  // Called when the user clicks the new auction button.
  doNewClick = (_evt: MouseEvent<HTMLButtonElement>): void => {
    this.props.onNewClick();
  };

  // Called when the user clicks on an auction item. index is the index of that
  // item in the auction list.
  doAuctionClick = (evt: MouseEvent<HTMLAnchorElement>, index: number): void => {
    evt.preventDefault();
    this.props.onAuctionClick(index);
  };
}
