import React, { Component, ChangeEvent, MouseEvent } from 'react';
import './NewAuction.css';


export type NewAuctionInfo = {
  /** Auction item name. */
  name: string;

  /** Auction item description. */
  description: string;

  /** Seller's name. */
  seller: string;

  /** max duration (in minutes) that can elapse after epoch. RI: minutes >= 1 */
  minutes: number;

  /** minimum (starting) bid allowed for this auction item. RI: minBid >= 1 */
  minBid: number;
};

type NewAuctionProps = {
  /** Tell the parent that an auction with the given info was started. */
  onStartClick: (info: NewAuctionInfo) => void;

  /** Tell the parent that back was clicked. */
  onBackClick: () => void;
};

type NewAuctionState = {
  /** Mirrors text in the auction item name textbox. */
  name: string;

  /** Mirrors text in the auction item description textbox. */
  description: string;

  /** Mirrors text in the seller's name textbox. */
  seller: string;

  /** Mirrors text in the max duration (in minutes) textbox. */
  minutes: string;

  /** Mirrors text in the minimum (starting) bid textbox. */
  minBid: string;

  /** Contents describe reason for entering error state if non-empty. */
  error: string;
};

/** Displays the UI for allowing the user to create a new auction. */
export class NewAuction extends Component<NewAuctionProps, NewAuctionState> {

  constructor(props: NewAuctionProps) {
    super(props);

    this.state = {name: "", description: "", seller: "", minutes: "60",
                  minBid: "1", error: ""};
  }

  render = (): JSX.Element => {
    return (
      <div>
        <h2>New Auction</h2>
        <div>
          <label htmlFor="name">Item Name: </label>
          <input id="name" type="text" value={this.state.name}
              onChange={this.doNameChange} required></input>
        </div>
        <div>
          <label htmlFor="description">Description: </label>
          <input id="description" type="text" value={this.state.description}
              onChange={this.doDescChange}></input>
        </div>
        <div>
          <label htmlFor="seller">Seller Name: </label>
          <input id="seller" type="text" value={this.state.seller}
              onChange={this.doSellerChange}></input>
        </div>
        <div>
          <label htmlFor="minutes">Minutes: </label>
          <input id="minutes" type="number" min="1" value={this.state.minutes}
              onChange={this.doMinutesChange}></input>
        </div>
        <div>
          <label htmlFor="minBid">Minimum Bid: </label>
          <input id="minBid" type="number" min="1" value={this.state.minBid}
              onChange={this.doMinBidChange}></input>
        </div>
        <button type="button" onClick={this.doStartClick}>Start</button>
        <button type="button" onClick={this.doBackClick}>Back</button>
        {this.renderError()}
      </div>);
  };

  // Returns an empty div iff not in error state. Otherwise, returns a div
  // that contains a user-friendly error message.
  renderError = (): JSX.Element => {
    if (this.state.error.length === 0) {
      return <div></div>;
    } else {
      // Note: See NewAuction.css for the styling of the error message.
      return (
        <div className="error-box">
          <p className="error-message"><b>Error</b>: {this.state.error}</p>
        </div>);
    }
  };

  // Called each time the user updates the text in the item name text box.
  doNameChange = (evt: ChangeEvent<HTMLInputElement>): void => {
    this.setState({name: evt.target.value, error: ""});
  };

  // Called each time the user updates the text in the description text box.
  doDescChange = (evt: ChangeEvent<HTMLInputElement>): void => {
    this.setState({description: evt.target.value, error: ""});
  };

  // Called each time the user updates the text in the seller's name text box.
  doSellerChange = (evt: ChangeEvent<HTMLInputElement>): void => {
    this.setState({seller: evt.target.value, error: ""});
  };

  // Called each time the user updates the text in the max duration text box.
  doMinutesChange = (evt: ChangeEvent<HTMLInputElement>): void => {
    this.setState({minutes: evt.target.value, error: ""});
  };

  // Called each time the user updates the text in the minimum (starting)
  // bid text box.
  doMinBidChange = (evt: ChangeEvent<HTMLInputElement>): void => {
    this.setState({minBid: evt.target.value, error: ""});
  };

  // Called when the user clicks on the start button. The auction will only
  // be started iff the auction details are valid. Otherwise, the error state
  // will be triggered.
  doStartClick = (_: MouseEvent<HTMLButtonElement>): void => {
    // Verify that the user entered all required information.
    if (this.state.name.trim().length === 0 ||
        this.state.description.trim().length === 0 ||
        this.state.seller.trim().length === 0 ||
        this.state.minutes.trim().length === 0 ||
        this.state.minBid.trim().length === 0) {
      this.setState({error: "a required field is missing."});
      return;
    }

    // Verify that minutes is a number.
    const minutes = parseFloat(this.state.minutes);
    if (isNaN(minutes) || minutes < 1 || Math.floor(minutes) !== minutes) {
      this.setState({error: "minutes is not a positive integer"});
      return;
    }

    // Ignore this request if the minutes or minBid are not numbers.
    const minBid = parseFloat(this.state.minBid);
    if (isNaN(minBid) || minBid < 1 || Math.floor(minBid) !== minBid) {
      this.setState({error: "min bid is not a positive integer"});
      return;
    }

    // Tell the parent to start this auction (adding it to the list).
    this.props.onStartClick({
        name: this.state.name,
        description: this.state.description,
        seller: this.state.seller,
        minutes, minBid });
  };

  // Called when the user clicks on the back button.
  doBackClick = (_: MouseEvent<HTMLButtonElement>): void => {
    this.props.onBackClick();
  };
}
